<?php
/**
 * Project:
 * Contenido Content Management System
 *
 * Description:
 * Implementation of Contenido_UrlBuilder to build front_content.php URL
 *
 * Requirements:
 * @con_php_req 5.0
 *
 * @package     Contenido
 * @subpackage  ModRewrite
 * @version     0.2
 * @author      Murat Purc
 * @copyright    Murat Purc 2008
 * @license     http://www.contenido.org/license/LIZENZ.txt
 * @link        http://www.4fb.de
 * @link        http://www.contenido.org
 */


if(!defined('CON_FRAMEWORK')) {
    die('Illegal call');
}


cInclude('classes', 'UrlBuilder/Contenido_UrlBuilder.class.php');
cInclude('classes', 'UrlBuilder/Contenido_UrlBuilderFactory.class.php');


/**
 * Class to build urls frontend urls for Advandced Mod Rewrite plugin.
 * Extends abstract Contenido_UrlBuilder class and implements singleton pattern.
 *
 * Usage:
 * <code>
 * $url = 'front_content.php?idart=123';
 * cInclude('plugins', 'mod_rewrite/classes/Contenido_UrlBuilder_MR.class.php');
 * $mrUrlBuilder = Contenido_UrlBuilder_MR::getInstance();
 * $mrUrlBuilder->buildUrl(array($url));
 * $newUrl = $mrUrlBuilder->getUrl();
 * </code>
 *
 * @author      Murat Purc <murat@purc.de>
 * @copyright    Murat Purc 2008
 * @package     Contenido
 * @subpackage  ModRewrite
 */
class Contenido_UrlBuilder_MR extends Contenido_UrlBuilder {

    /**
     * @var  Contenido_UrlBuilder_MR  Self instance
     */
    static private $_instance;

    /**
     * @var  string  Ampersant used for composong several parameter
     */
    private $_sAmp = '&amp;';

    /**
     * @var  bool  Is XHTML output?
     */
    private $_bIsXHTML = '&amp;';

    /**
     * @var  bool  Is mod rewrite enabled?
     */
    private $_bMREnabled = false;


    /**
     * @var  array  Mod Rewrite configuration
     */
    private $_aMrCfg = null;


    /**
     * Constructor, tries to set some member variables.
     */
    private function __construct() {
        $this->sHttpBasePath = '';
        if ($GLOBALS['cfg']['mod_rewrite']['use'] == 1) {
            $this->_aMrCfg     = $GLOBALS['cfg']['mod_rewrite'];
            $this->_bMREnabled = true;
            $this->_bIsXHTML   = (getEffectiveSetting('generator', 'xhtml', 'false') == 'false') ? false : true;
            $this->_sAmp       = ($this->_bIsXHTML) ? '&' : '&amp;';
        }
    }


    /**
     * Returns a instance of Contenido_UrlBuilder_MR
     *
     * @return  Contenido_UrlBuilder_MR
     */
    public static function getInstance() {
        if (self::$_instance == null) {
            self::$_instance = new Contenido_UrlBuilder_MR();
        }
        return self::$_instance;
    }


    /**
     * Builds a URL in front_content.php style.
     *
     * @param   array    $params  Parameter array, provides only following parameters:
     * <code>
     * $params[0] = 'front_content.php?idart=123...'
     * </code>
     * @param   boolean  $bUseAbsolutePath  Flag to use absolute path, not used at the moment
     * @return  string  new build url
     */
    public function buildUrl(array $params, $bUseAbsolutePath=false) {

        $url = (isset($params[0])) ? $params[0] : '';

        $urlDebug['in'] = $url;

        // cleanup any existing & entities
        $url = str_replace('&amp;', '&', $url);

        if ($this->_bMREnabled) {

            // now a little hack to replace occuring index_controller.php against
            // front_content.php. This happens using $auth->url()/$auth->purl()
            // which composes the URL using $PHP_SELF
            if (strpos($url, 'index_controller.php') !== false) {
                $url = str_replace('index_controller.php', 'front_content.php', $url);

                // self_url returns url including absoute path from root (/cms/foobar.php)
                // modify this...
                $path = $GLOBALS['cfgClient'][$GLOBALS['client']]['path']['htmlpath'];
                $comp = @parse_url($path);
                if (isset($comp['path']) && (strpos($url, $comp['path']) === 0)) {
                    $url = substr($url, strlen($comp['path']));
                }
            }

            $aUrl = ModRewrite::get_client_full_url_parts($url);
            if (preg_match("/^front_content\.php(.*|.+?)/i", $aUrl['url'], $arr_hits) == 1) {
                $url = $aUrl['htmlpath'] . self::_buildUrl($arr_hits[1]);
            } else {
                $GLOBALS['mpDebug']->addDebug($url, 'Contenido_UrlBuilder_MR::buildUrl() nomatch');
            }
        }

        // replace & against entity, if xhtml is to use
        if ($this->_bIsXHTML) {
            $url = str_replace('&', '&amp;', $url);
        }

        $urlDebug['out'] = $url;
        $GLOBALS['mpDebug']->addDebug($urlDebug, 'Contenido_UrlBuilder_MR::buildUrl() in -> out');

        $this->sUrl = $url;

    }


    /**
     * Private function to build the url by analyzing passed arguments (parameter value pairs)
     *
     * TODO: Too much code in one function -> is to redesign...
     *
     * @param   string  $args  Arguments
     * @return  string  New build url
     */
    private function _buildUrl($args='') {

        $sNewUrl        = '';
        $sCategories    = '';
        $sArticle       = '';
        $aArgs          = array();
        $sJoinParameter = '/';
        $sFileExtension = '';
        $aParts         = array();

        // only html files part
        if (ModRewrite::validate_setting_categories_as_html()) {
            $sJoinParameter = $this->_aMrCfg['category_seperator'];
            $sFileExtension = $this->_aMrCfg['file_extension'];
        }

        // check arguments ... and split
        $sAnchor = '';
        // check for anchor
        $aArgs = explode("#", $args);
        if (is_array($aArgs) && count($aArgs) > 1) {
            $args = $aArgs[0];
            $sAnchor = '#' . urlencode(urldecode($aArgs[1]));
        }

        $args = str_replace('?', '', $args);
        $args = str_replace('&amp;', '&', $args);

        // extract arguments into current scope
        parse_str($args);

        // some preparations to avoid too much checks in further code blocks
        $idcat        = (isset($idcat)) ? (int) $idcat : 0;
        $idart        = (isset($idart)) ? (int) $idart : 0;
        $idcatart     = (isset($idcatart)) ? (int) $idcatart : 0;
        $idartlang    = (isset($idartlang)) ? (int) $idartlang : 0;
        $changelang   = (isset($changelang)) ? (int) $changelang : 0;
        $changeclient = (isset($changeclient)) ? (int) $changeclient : 0;

        // get additional non contenido parameters
        $sAdditionalParams = '';
        $aAdditionalParams = array();
        $aParamPairs       = split('&', $args);

        $aIgnoredParams = array(
            'idcat', 'idart', 'lang', 'client', 'idcatart', 'changelang', 'changeclient', 'idartlang'
        );

        foreach ($aParamPairs as $sPair) {
            $bFoundBad = false;
            $arr_param = explode('=', $sPair);

            foreach ($aIgnoredParams as $sKey) {
                if ($sKey == strtolower(trim($arr_param[0]))) {
                    $bFoundBad = true;
                    break;
                }
            }

            if ($bFoundBad == false) {
                $aAdditionalParams[] = urlencode(urldecode($arr_param[0])) . '=' . urlencode(urldecode($arr_param[1]));
            }
        }

        if (count($aAdditionalParams) > 0) {
            $sAdditionalParams = '?' . implode($this->_sAmp, $aAdditionalParams);
        }

        $idlang   = ($changelang > 0) ? $changelang : $GLOBALS['lang'];
        $idclient = ($changeclient > 0) ? $changeclient : $GLOBALS['client'];

        // set client?
        if ($this->_aMrCfg['use_client'] == 1) {
            if ($this->_aMrCfg['use_client_name'] == 1) {
                $aParts[] = urlencode(ModRewrite::get_client_name($idclient));
            } else {
                $aParts[] = $idclient;
            }
        }

        // set language?
        if ($this->_aMrCfg['use_language'] == 1) {
            if ($this->_aMrCfg['use_language_name'] == 1) {
                $aParts[] = urlencode(ModRewrite::get_language_name($idlang));
            } else {
                $aParts[] = $idlang;
            }
        }

        // define rootpath ...
        $sNewUrl = $this->_aMrCfg['rootdir'];

        // handle idcatart
        if ($idcatart > 0 && $idcat == 0 && $idart == 0 && $idartlang == 0) {
            $sql = "SELECT idcat, idart FROM " . $GLOBALS['cfg']['tab']['cat_art'] . " WHERE idcatart='$idcatart'";
            if ($aData = mr_query_n_next_record($sql)) {
                $idcat = $aData['idcat'];
                $idart = $aData['idart'];
            }
        }

        // check if article id is given and set article url
        if ($idart > 0 && $idartlang == 0) {
            $sArticle = ModRewrite::get_art_websafename($idart, $idlang);
        } elseif ($idartlang > 0 && $idart == 0) {
            // handle idartlang
            $sArticle = ModRewrite::get_art_lang_websafename($idartlang);
        }

        // check if only article id is given, cat id have to rebuild
        if ($idart > 0 && $idcat == 0 && $idartlang == 0) {
            $sql = "SELECT idcat FROM " . $GLOBALS['cfg']['tab']['cat_art'] . " WHERE idart='$idart'";
            if ($aData = mr_query_n_next_record($sql)) {
                $idcat = $aData['idcat'];
            }
        }

        // idartlang is given
        if ($idartlang > 0 && $idcat == 0 && $idart == 0) {
            $sql = "SELECT ca.idcat
                    FROM " . $GLOBALS['cfg']['tab']['art_lang'] . " al
                    LEFT JOIN " . $GLOBALS['cfg']['tab']['cat_art'] . " ca ON al.idart = ca.idart
                    WHERE al.idartlang='$idartlang'";
            if ($aData = mr_query_n_next_record($sql)) {
                $idcat = $aData['idcat'];
            }
        }

        // we have category but no article name and start article name should be added to url
        if ($idcat > 0 && $sArticle == '' && $this->_aMrCfg['add_startart_name_to_url']) {
            if ($this->_aMrCfg['default_startart_name'] == '') {

                // no default article name is configured get startarticle
                cInclude('classes', 'class.article.php');
                $artColl = new ArticleCollection(array('idcat' => $idcat, 'start' => 1));
                if ($artItem = $artColl->startArticle()) {
                    $sArticle = ModRewrite::get_art_websafename($artItem->get('idart'), $idlang);
                }

            } else {

                // use default start article name
                $sArticle = $this->_aMrCfg['default_startart_name'];
            }
        }

        if ($sArticle) {
            $sFileExtension = $this->_aMrCfg['file_extension'];
        }

        // ok build dir list, if idcat found ...
        if ($idcat > 0) {

            $sCategories = ModRewrite::build_recursiv_path ($idcat, $idlang);

            // check start directory settings
            if ($this->_aMrCfg['startfromroot'] == 0) {
                // splitt string in array
                $aCategories = explode($sJoinParameter, $sCategories);

                // remove first category
                $sFirstCat = array_shift($aCategories);

                // implode array with categories to new string
                $sCategories = implode($sJoinParameter, $aCategories);
            }

            if (strlen($sCategories) > 0) {
                $aParts[] = $sCategories;
            }
        }
        $sParts = implode($sJoinParameter, $aParts);

        // only html files part
        if (strlen($sParts) > 0 && strlen($sArticle) > 0
            && ModRewrite::validate_setting_categories_as_html()) {
            $sParts .= $this->_aMrCfg['article_seperator'];
        } elseif ((int)$this->_aMrCfg['use_categories_as_html_file'] != 1 && strlen($sParts) > 0) {
            $sParts .= $sJoinParameter;
        }

        // using lowercase uri?
        if ($this->_aMrCfg['use_lowercase_uri'] == 1) {
            $sFullUrl = $sNewUrl . strtolower($sParts . $sArticle . $sFileExtension) . $sAdditionalParams . $sAnchor;
        } else {
            $sFullUrl = $sNewUrl . $sParts . $sArticle . $sFileExtension . $sAdditionalParams . $sAnchor;
        }

        // remove double slashes
        while (eregi($sJoinParameter.$sJoinParameter, $sFullUrl)) {
            $sFullUrl = eregi_replace($sJoinParameter . $sJoinParameter, $sJoinParameter, $sFullUrl);
        }

        if (substr($sFullUrl, -2) == '?=') {
            $sFullUrl = substr_replace($sFullUrl, '', -2);
        }

        return $sFullUrl;

    }

}
